# -*- coding: iso-8859-1 -*-
"""
    MoinMoin - PageListPlus

    print a list of pages whose title matches the search term

    @copyright: 2010 Paul Boddie <paul@boddie.org.uk>
                2001-2003 Juergen Hermann <jh@web.de>
                2003-2008 MoinMoin:ThomasWaldmann
                2008 MoinMoin:ReimarBauer
    @license: GNU GPL, see COPYING for details.
"""

Dependencies = ["pages"]
from MoinMoin import wikiutil
import ImprovedMoinSearch
import re

format_pattern = re.compile("""format:(?P<quote>['"])(?P<format>.*?)(?P=quote)""")
paging_pattern = re.compile("""paging:(?P<paging>on|off|yes|no|true|false)""")

def extract_option(query, pattern, group):

    """
    Remove from 'query', using the given regular expression 'pattern' object,
    a particular form of argument text, accessing such text through the group
    with the given 'group' name, and returning a tuple of the form
    (edited_query, value_of_group).
    """

    m = pattern.search(query)
    if m:
        value = m.group(group)
        query = query[:m.start()] + query[m.end():]
    else:
        value = None
    return query, value

# Macro functions.

def execute(macro, args):

    """
    Execute the 'macro' with the given 'args': 
    """

    request = macro.request
    formatter = macro.formatter
    _ = macro._

    # If called with empty or no argument, default to regex search for .+, the full page list.

    needle = wikiutil.get_unicode(request, args, 'needle', u'regex:.+')

    # Extract format and paging options.

    needle, format = extract_option(needle, format_pattern, "format")
    needle, paging = extract_option(needle, paging_pattern, "paging")

    # With whitespace argument, return same error message as FullSearch.

    if not needle.strip():
        err = _('Please use a more selective search term instead of {{{"%s"}}}', wiki=True) % needle
        return '<span class="error">%s</span>' % err

    # Return a title search for needle, sorted by name.

    try:
        pages = ImprovedMoinSearch.getSearchResultPages(request, needle, titlesearch=1, sort='page_name')

        # Get any paging information from the request.

        page_from = int(request.form.get("from", ["0"])[0])

        # Format the results.

        ret = ImprovedMoinSearch.formatResultPages(request, formatter, pages, paging, format, page_from)

    except ValueError:

        # Same error as in MoinMoin/action/fullsearch.py, keep it that way!

        ret = ''.join([formatter.text('<<PageListPlus('),
                      _('Your search query {{{"%s"}}} is invalid. Please refer to '
                        'HelpOnSearching for more information.', wiki=True,
                        percent=True) % wikiutil.escape(needle),
                      formatter.text(')>>')])

    return ret

# vim: tabstop=4 expandtab shiftwidth=4
